@@ -16,23 +16,7 @@
  * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
-#include "../miral/basic_window_manager.h"
-
-#include <miral/canonical_window_manager.h>
-
-#include <mir/frontend/surface_id.h>
-#include <mir/scene/surface_creation_parameters.h>
-#include <mir/shell/display_layout.h>
-#include <mir/shell/persistent_surface_store.h>
-
-#include <mir/test/doubles/stub_session.h>
-#include <mir/test/doubles/stub_surface.h>
-#include <mir/test/fake_shared.h>
-
-#include <gtest/gtest.h>
-#include <gmock/gmock.h>
-
-#include <atomic>
+#include "test_window_manager_tools.h"
 
 using namespace miral;
 using namespace testing;
@@ -40,90 +24,6 @@ namespace mt = mir::test;
 
 namespace
 {
-struct StubFocusController : mir::shell::FocusController
-{
-    void focus_next_session() override {}
-
-    auto focused_session() const -> std::shared_ptr<mir::scene::Session> override { return {}; }
-
-    void set_focus_to(
-        std::shared_ptr<mir::scene::Session> const& /*focus_session*/,
-        std::shared_ptr<mir::scene::Surface> const& /*focus_surface*/) override {}
-
-    auto focused_surface() const -> std::shared_ptr<mir::scene::Surface> override { return {}; }
-
-    void raise(mir::shell::SurfaceSet const& /*surfaces*/) override {}
-
-    virtual auto surface_at(mir::geometry::Point /*cursor*/) const -> std::shared_ptr<mir::scene::Surface> override
-        { return {}; }
-};
-
-struct StubDisplayLayout : mir::shell::DisplayLayout
-{
-    void clip_to_output(mir::geometry::Rectangle& /*rect*/) override {}
-
-    void size_to_output(mir::geometry::Rectangle& /*rect*/) override {}
-
-    bool place_in_output(mir::graphics::DisplayConfigurationOutputId /*id*/, mir::geometry::Rectangle& /*rect*/) override
-        { return false; }
-};
-
-struct StubPersistentSurfaceStore : mir::shell::PersistentSurfaceStore
-{
-    Id id_for_surface(std::shared_ptr<mir::scene::Surface> const& /*surface*/) override { return {}; }
-
-    auto surface_for_id(Id const& /*id*/) const -> std::shared_ptr<mir::scene::Surface> override { return {}; }
-};
-
-struct StubSurface : mir::test::doubles::StubSurface
-{
-    StubSurface(Point top_left, Size size) : top_left_{top_left}, size_{size} {}
-
-    Point top_left() const override { return top_left_; }
-    void move_to(Point const& top_left) override { top_left_ = top_left; }
-
-    Size size() const override { return  size_; }
-    void resize(Size const& size) override { size_ = size; }
-
-    Point top_left_;
-    Size size_;
-};
-
-struct StubStubSession : mir::test::doubles::StubSession
-{
-    mir::frontend::SurfaceId create_surface(
-        mir::scene::SurfaceCreationParameters const& params,
-        std::shared_ptr<mir::frontend::EventSink> const& /*sink*/) override
-    {
-        auto id = mir::frontend::SurfaceId{next_surface_id.fetch_add(1)};
-        auto surface = std::make_shared<StubSurface>(params.top_left, params.size);
-        surfaces[id] = surface;
-        return id;
-    }
-
-    std::shared_ptr<mir::scene::Surface> surface(mir::frontend::SurfaceId surface) const override
-    {
-        return surfaces.at(surface);
-    }
-
-private:
-    std::atomic<int> next_surface_id;
-    std::map<mir::frontend::SurfaceId, std::shared_ptr<mir::scene::Surface>> surfaces;
-};
-
-struct MockWindowManagerPolicy : CanonicalWindowManagerPolicy
-{
-    using CanonicalWindowManagerPolicy::CanonicalWindowManagerPolicy;
-
-    bool handle_touch_event(MirTouchEvent const* /*event*/) override { return false; }
-    bool handle_pointer_event(MirPointerEvent const* /*event*/) override { return false; }
-    bool handle_keyboard_event(MirKeyboardEvent const* /*event*/) override { return false; }
-
-    MOCK_METHOD1(advise_new_window, void (WindowInfo const& window_info));
-    MOCK_METHOD2(advise_move_to, void(WindowInfo const& window_info, Point top_left));
-    MOCK_METHOD2(advise_resize, void(miral::WindowInfo const& window_info, Size const& new_size));
-};
-
 X const display_left{0};
 Y const display_top{0};
 Width  const display_width{640};
@@ -133,15 +33,6 @@ Rectangle const display_area{{display_left, display_top}, {display_width, displa
 
 auto const null_window = Window{};
 
-auto create_surface(std::shared_ptr<mir::scene::Session> const& session, mir::scene::SurfaceCreationParameters const& params)
--> mir::frontend::SurfaceId
-{
-    // This type is Mir-internal, I hope we don't need to create it here
-    std::shared_ptr<mir::frontend::EventSink> const sink;
-
-    return session->create_surface(params, sink);
-}
-
 mir::shell::SurfaceSpecification edge_attachment(Rectangle const& aux_rect, MirEdgeAttachment attachment)
 {
     mir::shell::SurfaceSpecification result;
@@ -150,36 +41,18 @@ mir::shell::SurfaceSpecification edge_attachment(Rectangle const& aux_rect, MirE
     return result;
 }
 
-struct WindowPlacement : testing::Test
+struct WindowPlacement : TestWindowManagerTools
 {
-    StubFocusController focus_controller;
-    StubDisplayLayout display_layout;
-    StubPersistentSurfaceStore persistent_surface_store;
-    std::shared_ptr<StubStubSession> session{std::make_shared<StubStubSession>()};
-
-    MockWindowManagerPolicy* window_manager_policy;
-    BasicWindowManager basic_window_manager{
-        &focus_controller,
-        mt::fake_shared(display_layout),
-        mt::fake_shared(persistent_surface_store),
-        [this](WindowManagerTools const& tools) -> std::unique_ptr<WindowManagementPolicy> 
-            {
-                auto policy = std::make_unique<MockWindowManagerPolicy>(tools);
-                window_manager_policy = policy.get();
-                return std::move(policy);
-            }
-        };
-
     Size const initial_parent_size{600, 400};
     Size const initial_child_size{300, 300};
     Rectangle const rectangle_away_from_rhs{{20, 20}, {20, 20}};
-    Rectangle const rectangle_near_rhs{{600, 20}, {10, 20}};
+    Rectangle const rectangle_near_rhs{{590, 20}, {10, 20}};
     Rectangle const rectangle_away_from_bottom{{20, 20}, {20, 20}};
-    Rectangle const rectangle_near_bottom{{20, 400}, {20, 20}};
+    Rectangle const rectangle_near_bottom{{20, 380}, {20, 20}};
     Rectangle const rectangle_near_both_sides{{0, 20}, {600, 20}};
-    Rectangle const rectangle_near_both_sides_and_bottom{{0, 400}, {600, 20}};
-    Rectangle const rectangle_near_all_sides{{0, 20}, {600, 400}};
-    Rectangle const rectangle_near_both_bottom_right{{400, 400}, {200, 20}};
+    Rectangle const rectangle_near_both_sides_and_bottom{{0, 380}, {600, 20}};
+    Rectangle const rectangle_near_all_sides{{0, 20}, {600, 380}};
+    Rectangle const rectangle_near_both_bottom_right{{400, 380}, {200, 20}};
 
     Window parent;
     Window child;
