@@ -59,6 +59,7 @@ struct miral::WindowSpecification::Self
     mir::optional_value<std::vector<Rectangle>> input_shape;
     mir::optional_value<InputReceptionMode> input_mode;
     mir::optional_value<MirShellChrome> shell_chrome;
+    mir::optional_value<MirPointerConfinementState> confine_pointer;
 };
 
 miral::WindowSpecification::Self::Self(mir::shell::SurfaceSpecification const& spec) :
@@ -90,6 +91,9 @@ miral::WindowSpecification::Self::Self(mir::shell::SurfaceSpecification const& s
     input_shape(spec.input_shape),
     input_mode(),
     shell_chrome(spec.shell_chrome)
+#if MIRAL_MIR_DEFINES_POINTER_CONFINEMENT
+    ,confine_pointer(spec.confine_pointer)
+#endif
 {
 #if MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 25, 0)
     if (spec.aux_rect_placement_offset_x.is_set() && spec.aux_rect_placement_offset_y.is_set())
@@ -236,6 +240,9 @@ miral::WindowSpecification::Self::Self(mir::scene::SurfaceCreationParameters con
     input_shape(params.input_shape),
     input_mode(static_cast<InputReceptionMode>(params.input_mode)),
     shell_chrome(params.shell_chrome)
+#if MIRAL_MIR_DEFINES_POINTER_CONFINEMENT
+    ,confine_pointer(params.confine_pointer)
+#endif
 {
 #if MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 25, 0)
     if (params.aux_rect_placement_offset_x.is_set() && params.aux_rect_placement_offset_y.is_set())
@@ -304,6 +311,9 @@ void miral::WindowSpecification::Self::update(mir::scene::SurfaceCreationParamet
     copy_if_set(params.input_shape, input_shape);
     copy_if_set(params.input_mode, input_mode);
     copy_if_set(params.shell_chrome, shell_chrome);
+#if MIRAL_MIR_DEFINES_POINTER_CONFINEMENT
+    copy_if_set(params.confine_pointer, confine_pointer);
+#endif
 
 #if MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 25, 0)
     copy_if_set(params.placement_hints, placement_hints);
@@ -472,6 +482,11 @@ auto miral::WindowSpecification::shell_chrome() const -> mir::optional_value<Mir
     return self->shell_chrome;
 }
 
+auto miral::WindowSpecification::confine_pointer() const -> mir::optional_value<MirPointerConfinementState> const&
+{
+    return self->confine_pointer;
+}
+
 auto miral::WindowSpecification::top_left() -> mir::optional_value<Point>&
 {
     return self->top_left;
@@ -482,11 +497,6 @@ auto miral::WindowSpecification::size() -> mir::optional_value<Size>&
     return self->size;
 }
 
-auto miral::WindowSpecification::pixel_format() -> mir::optional_value<MirPixelFormat>&
-{
-    return self->pixel_format;
-}
-
 auto miral::WindowSpecification::name() -> mir::optional_value<std::string>&
 {
     return self->name;
@@ -596,3 +606,8 @@ auto miral::WindowSpecification::shell_chrome() -> mir::optional_value<MirShellC
 {
     return self->shell_chrome;
 }
+
+auto miral::WindowSpecification::confine_pointer() -> mir::optional_value<MirPointerConfinementState>&
+{
+    return self->confine_pointer;
+}
